import argparse

import nvgpu
from nvgpu.list_gpus import pretty_list_gpus

def parse_args():
    parser = argparse.ArgumentParser(description='NVIDIA GPU tools.')
    subparsers = parser.add_subparsers()
    subparsers.required = True
    subparsers.dest = 'command'
    parser_available = subparsers.add_parser('available')
    parser_available.add_argument('-l', '--limit', default=None, type=int,
        help='Max number of GPUs')
    parser_list = subparsers.add_parser('list')

    return parser.parse_args()

def main():
    args = parse_args()
    if args.command == 'available':
        gpus = nvgpu.available_gpus()
        if args.limit:
            gpus = gpus[:args.limit]
        print(','.join(gpus))
    elif args.command == 'list':
        pretty_list_gpus()

if __name__ == '__main__':
    main()



import re
import subprocess


def gpu_info():
    gpus = [line for line in subprocess.check_output(['nvidia-smi', '-L']).split('\n') if line]
    gpu_infos = [re.match('GPU ([0-9]+): ([^(]+) \(UUID: ([^)]+)\)', gpu).groups() for gpu in gpus]
    gpu_infos = [dict(zip(['index', 'type', 'uuid'], info)) for info in gpu_infos]
    gpu_count = len(gpus)

    lines = subprocess.check_output(['nvidia-smi']).split('\n')
    selected_lines = lines[7:7 + 3 * gpu_count]
    for i in range(gpu_count):
        mem_used, mem_total = [int(m.strip().replace('MiB', '')) for m in
                               selected_lines[3 * i + 1].split('|')[2].strip().split('/')]
        gpu_infos[i]['mem_used'] = mem_used
        gpu_infos[i]['mem_total'] = mem_total
        gpu_infos[i]['mem_used_percent'] = 100. * mem_used / mem_total

    return gpu_infos


def available_gpus(max_used_percent=20.):
    return [gpu['index'] for gpu in gpu_info() if gpu['mem_used_percent'] <= max_used_percent]



from __future__ import print_function

import arrow
import pandas as pd
import psutil
import pynvml as nv
from tabulate import tabulate
from termcolor import colored

from nvgpu.nvml import nvml_context

def device_status(device_index):
    handle = nv.nvmlDeviceGetHandleByIndex(device_index)
    device_name = nv.nvmlDeviceGetName(handle)
    nv_procs = nv.nvmlDeviceGetComputeRunningProcesses(handle)
    utilization = nv.nvmlDeviceGetUtilizationRates(handle).gpu
    pids = []
    users = []
    dates = []
    for nv_proc in nv_procs:
        pid = nv_proc.pid
        pids.append(pid)
        try:
            proc = psutil.Process(pid)
            users.append(proc.username())
            dates.append(proc.create_time())
        except psutil._exceptions.NoSuchProcess:
            users.append('?')
    return {
        'type': device_name,
        'is_available': len(pids) == 0,
        'pids': ','.join([str(pid) for pid in pids]),
        'users': ','.join(users),
        'running_since': arrow.get(min(dates)).humanize() if len(dates) > 0 else None,
        'utilization': utilization
    }

def device_table():
    with nvml_context():
        device_count = nv.nvmlDeviceGetCount()
        rows = [device_status(device_index) for device_index in range(device_count)]
        df = pd.DataFrame(rows, columns=['is_available', 'type', 'utilization', 'users', 'running_since', 'pids'])
        return df

def pretty_list_gpus():
    df = device_table()
    df['status'] = df['is_available'].apply(lambda a: '[ ]' if a else '[~]')
    df['util.'] = df['utilization'].apply(lambda u: '%02s %%' % u)
    for col in ['status', 'type', 'util.', 'users', 'running_since', 'pids']:
        df[col] = [colored(row[col], 'green') if row['is_available'] else colored(row[col], 'red') for i, row in df.iterrows()]
    df = df[['status', 'type', 'util.', 'users', 'running_since', 'pids']]
    print(tabulate(df, headers='keys'))

if __name__ == '__main__':
    pretty_list_gpus()



from setuptools import setup

setup(name='nvgpu',
      version='0.3',
      description='NVIDIA GPU tools',
      url='https://github.com/rossumai/nvgpu',
      author='Bohumir Zamecnik, Rossum',
      author_email='bohumir.zamecnik@rossum.ai',
      license='MIT',
      packages=['nvgpu'],
      zip_safe=False,
      install_requires=[
        'arrow',
        'nvidia-ml-py',
        'pandas',
        'psutil',
        'termcolor',
        'tabulate',
      ],
      setup_requires=['setuptools-markdown'],
      long_description_markdown_filename='README.md',
      # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          # How mature is this project? Common values are
          #   3 - Alpha
          #   4 - Beta
          #   5 - Production/Stable
          'Development Status :: 3 - Alpha',

          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',

          'License :: OSI Approved :: MIT License',

          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',

          'Operating System :: POSIX :: Linux',
      ],
      entry_points={
          'console_scripts': [
              'nvgpu = nvgpu.__main__:main'
          ]
      },)
